@@ -11,12 +11,12 @@ use std::thread;
 
 use crate::config::{Color, Config, EmitMode, FileName, NewlineStyle, ReportTactic};
 use crate::formatting::{ReportedErrors, SourceFile};
-use crate::is_nightly_channel;
 use crate::rustfmt_diff::{make_diff, print_diff, DiffLine, Mismatch, ModifiedChunk, OutputWriter};
 use crate::source_file;
-use crate::{FormatReport, FormatReportFormatterBuilder, Input, Session};
+use crate::{is_nightly_channel, FormatReport, FormatReportFormatterBuilder, Input, Session};
 
 mod configuration_snippet;
+mod parser;
 
 const DIFF_CONTEXT_SIZE: usize = 3;
 
@@ -40,6 +40,7 @@ const SKIP_FILE_WHITE_LIST: &[&str] = &[
     "cfg_mod/bar.rs",
     "cfg_mod/foo.rs",
     "cfg_mod/wasm32.rs",
+    "skip/foo.rs",
 ];
 
 fn init_log() {
@@ -132,10 +133,8 @@ fn verify_config_used(path: &Path, config_name: &str) {
                     .map(Result::unwrap)
                     .take_while(|l| l.starts_with("//"))
                     .any(|l| l.starts_with(&format!("// rustfmt-{}", config_name))),
-                format!(
-                    "config option file {} does not contain expected config name",
-                    path.display()
-                )
+                "config option file {} does not contain expected config name",
+                path.display()
             );
         }
     }
@@ -371,6 +370,44 @@ fn self_tests() {
     );
 }
 
+#[test]
+fn format_files_find_new_files_via_cfg_if() {
+    init_log();
+    run_test_with(&TestSetting::default(), || {
+        // To repro issue-4656, it is necessary that these files are parsed
+        // as a part of the same session (hence this separate test runner).
+        let files = vec![
+            Path::new("tests/source/issue-4656/lib2.rs"),
+            Path::new("tests/source/issue-4656/lib.rs"),
+        ];
+
+        let config = Config::default();
+        let mut session = Session::<io::Stdout>::new(config, None);
+
+        let mut write_result = HashMap::new();
+        for file in files {
+            assert!(file.exists());
+            let result = session.format(Input::File(file.into())).unwrap();
+            assert!(!session.has_formatting_errors());
+            assert!(!result.has_warnings());
+            let mut source_file = SourceFile::new();
+            mem::swap(&mut session.source_file, &mut source_file);
+
+            for (filename, text) in source_file {
+                if let FileName::Real(ref filename) = filename {
+                    write_result.insert(filename.to_owned(), text);
+                }
+            }
+        }
+        assert_eq!(
+            3,
+            write_result.len(),
+            "Should have uncovered an extra file (format_me_please.rs) via lib.rs"
+        );
+        assert!(handle_result(write_result, None).is_ok());
+    });
+}
+
 #[test]
 fn stdin_formatting_smoke_test() {
     init_log();
@@ -845,6 +882,7 @@ fn rustfmt() -> PathBuf {
     me.push("rustfmt");
     assert!(
         me.is_file() || me.with_extension("exe").is_file(),
+        "{}",
         if cfg!(release) {
             "no rustfmt bin, try running `cargo build --release` before testing"
         } else {
